=== modified file 'bzrlib/smart/protocol.py'
--- bzrlib/smart/protocol.py	2007-04-24 12:20:09 +0000
+++ bzrlib/smart/protocol.py	2007-04-24 12:50:50 +0000
@@ -133,11 +133,11 @@
     def _send_response(self, response):
         """Send a smart server response down the output stream."""
         assert not self._finished, 'response already sent'
-        response.is_successful()
         args = response.args
         body = response.body
         self._finished = True
         self._write_protocol_version()
+        self._write_success_or_failure_prefix(response)
         self._write_func(_encode_tuple(args))
         if body is not None:
             assert isinstance(body, str), 'body must be a str'
@@ -150,6 +150,14 @@
         Version one doesn't send protocol versions.
         """
 
+    def _write_success_or_failure_prefix(self, response):
+        """Write the protocol specific success/failure prefix.
+
+        For SmartServerRequestProtocolOne this is omitted but we
+        call is_successful to ensure that the response is valid.
+        """
+        response.is_successful()
+
     def next_read_size(self):
         if self._finished:
             return 0
@@ -165,6 +173,13 @@
     This prefixes responses with the protocol version: "2\n".
     """
 
+    def _write_success_or_failure_prefix(self, response):
+        """Write the protocol specific success/failure prefix."""
+        if response.is_successful():
+            self._write_func('success\n')
+        else:
+            self._write_func('failed\n')
+
     def _write_protocol_version(self):
         r"""Write any prefixes this protocol requires.
         
@@ -342,13 +357,17 @@
 
     def _recv_tuple(self):
         """Receive a tuple from the medium request."""
+        return _decode_tuple(self._recv_line())
+
+    def _recv_line(self):
+        """Read an entire line from the medium request."""
         line = ''
         while not line or line[-1] != '\n':
             # TODO: this is inefficient - but tuples are short.
             new_char = self._request.read_bytes(1)
             line += new_char
             assert new_char != '', "end of file reading from server."
-        return _decode_tuple(line)
+        return line
 
     def query_version(self):
         """Return protocol version number of the server."""
@@ -389,6 +408,11 @@
         version = self._request.read_bytes(2)
         if version != SmartClientRequestProtocolTwo._version_string:
             raise errors.SmartProtocolError('bad protocol marker %r' % version)
+        response_status = self._recv_line()
+        if response_status not in ('success\n', 'failed\n'):
+            raise errors.SmartProtocolError(
+                'bad protocol status %r' % response_status)
+        self.response_status = response_status == 'success\n'
         return SmartClientRequestProtocolOne.read_response_tuple(self, expect_body)
 
     def _write_protocol_version(self):

=== modified file 'bzrlib/tests/test_smart_transport.py'
--- bzrlib/tests/test_smart_transport.py	2007-04-24 12:20:09 +0000
+++ bzrlib/tests/test_smart_transport.py	2007-04-24 12:50:50 +0000
@@ -1566,7 +1566,7 @@
         self.assertEqual('abc', smart_protocol.in_buffer)
         smart_protocol.accept_bytes('\n')
         self.assertEqual(
-            "2\nerror\x01Generic bzr smart protocol error: bad request 'abc'\n",
+            "2\nfailed\nerror\x01Generic bzr smart protocol error: bad request 'abc'\n",
             out_stream.getvalue())
         self.assertTrue(smart_protocol.has_dispatched)
         self.assertEqual(0, smart_protocol.next_read_size())
@@ -1590,7 +1590,7 @@
                 out_stream.write)
         smart_protocol.accept_bytes('readv\x01foo\n3\n3,3done\n')
         self.assertEqual(0, smart_protocol.next_read_size())
-        self.assertEqual('2\nreadv\n3\ndefdone\n', out_stream.getvalue())
+        self.assertEqual('2\nsuccess\nreadv\n3\ndefdone\n', out_stream.getvalue())
         self.assertEqual('', smart_protocol.excess_buffer)
         self.assertEqual('', smart_protocol.in_buffer)
 
@@ -1599,7 +1599,7 @@
         smart_protocol = protocol.SmartServerRequestProtocolTwo(
             None, out_stream.write)
         smart_protocol.accept_bytes('hello\nhello\n')
-        self.assertEqual("2\nok\x012\n", out_stream.getvalue())
+        self.assertEqual("2\nsuccess\nok\x012\n", out_stream.getvalue())
         self.assertEqual("hello\n", smart_protocol.excess_buffer)
         self.assertEqual("", smart_protocol.in_buffer)
 
@@ -1619,7 +1619,7 @@
         smart_protocol = protocol.SmartServerRequestProtocolTwo(
             None, out_stream.write)
         smart_protocol.accept_bytes('hello\n')
-        self.assertEqual("2\nok\x012\n", out_stream.getvalue())
+        self.assertEqual("2\nsuccess\nok\x012\n", out_stream.getvalue())
         smart_protocol.accept_bytes('2\nhel')
         self.assertEqual("2\nhel", smart_protocol.excess_buffer)
         smart_protocol.accept_bytes('lo\n')
@@ -1636,11 +1636,29 @@
 
     def test__send_response_errors_with_base_response(self):
         """Ensure that only the Successful/Failed subclasses are used."""
-        smart_protocol = protocol.SmartServerRequestProtocolOne(
+        smart_protocol = protocol.SmartServerRequestProtocolTwo(
             None, lambda x: None)
         self.assertRaises(AttributeError, smart_protocol._send_response,
             request.SmartServerResponse(('x',)))
 
+    def test__send_response_includes_failure_marker(self):
+        """FailedSmartServerResponse have 'failed\n' after the version."""
+        out_stream = StringIO()
+        smart_protocol = protocol.SmartServerRequestProtocolTwo(
+            None, out_stream.write)
+        smart_protocol._send_response(
+            request.FailedSmartServerResponse(('x',)))
+        self.assertEqual('2\nfailed\nx\n', out_stream.getvalue())
+
+    def test__send_response_includes_success_marker(self):
+        """SuccessfulSmartServerResponse have 'success\n' after the version."""
+        out_stream = StringIO()
+        smart_protocol = protocol.SmartServerRequestProtocolTwo(
+            None, out_stream.write)
+        smart_protocol._send_response(
+            request.SuccessfulSmartServerResponse(('x',)))
+        self.assertEqual('2\nsuccess\nx\n', out_stream.getvalue())
+
     def test_query_version(self):
         """query_version on a SmartClientProtocolTwo should return a number.
         
@@ -1651,7 +1669,7 @@
         # accept_bytes(tuple_based_encoding_of_hello) and reads and parses the
         # response of tuple-encoded (ok, 1).  Also, seperately we should test
         # the error if the response is a non-understood version.
-        input = StringIO('2\nok\x012\n')
+        input = StringIO('2\nsuccess\nok\x012\n')
         output = StringIO()
         client_medium = medium.SmartSimplePipesClientMedium(input, output)
         request = client_medium.get_request()
@@ -1662,12 +1680,12 @@
         # protocol.call() can get back an empty tuple as a response. This occurs
         # when the parsed line is an empty line, and results in a tuple with
         # one element - an empty string.
-        self.assertServerToClientEncoding('2\n\n', ('', ), [(), ('', )])
+        self.assertServerToClientEncoding('2\nsuccess\n\n', ('', ), [(), ('', )])
 
     def test_client_call_three_element_response(self):
         # protocol.call() can get back tuples of other lengths. A three element
         # tuple should be unpacked as three strings.
-        self.assertServerToClientEncoding('2\na\x01b\x0134\n', ('a', 'b', '34'),
+        self.assertServerToClientEncoding('2\nsuccess\na\x01b\x0134\n', ('a', 'b', '34'),
             [('a', 'b', '34')])
 
     def test_client_call_with_body_bytes_uploads(self):
@@ -1694,11 +1712,22 @@
         smart_protocol.call_with_body_readv_array(('foo', ), [(1,2),(5,6)])
         self.assertEqual(expected_bytes, output.getvalue())
 
+    def test_client_read_response_tuple_sets_response_status(self):
+        server_bytes = "2\nsuccess\nok\n"
+        input = StringIO(server_bytes)
+        output = StringIO()
+        client_medium = medium.SmartSimplePipesClientMedium(input, output)
+        request = client_medium.get_request()
+        smart_protocol = protocol.SmartClientRequestProtocolTwo(request)
+        smart_protocol.call('foo')
+        smart_protocol.read_response_tuple(False)
+        self.assertEqual(True, smart_protocol.response_status)
+
     def test_client_read_body_bytes_all(self):
         # read_body_bytes should decode the body bytes from the wire into
         # a response.
         expected_bytes = "1234567"
-        server_bytes = "2\nok\n7\n1234567done\n"
+        server_bytes = "2\nsuccess\nok\n7\n1234567done\n"
         input = StringIO(server_bytes)
         output = StringIO()
         client_medium = medium.SmartSimplePipesClientMedium(input, output)
@@ -1715,7 +1744,7 @@
         # LengthPrefixedBodyDecoder that is already well tested - we can skip
         # that.
         expected_bytes = "1234567"
-        server_bytes = "2\nok\n7\n1234567done\n"
+        server_bytes = "2\nsuccess\nok\n7\n1234567done\n"
         input = StringIO(server_bytes)
         output = StringIO()
         client_medium = medium.SmartSimplePipesClientMedium(input, output)
@@ -1732,7 +1761,7 @@
         # cancelling the expected body needs to finish the request, but not
         # read any more bytes.
         expected_bytes = "1234567"
-        server_bytes = "2\nok\n7\n1234567done\n"
+        server_bytes = "2\nsuccess\nok\n7\n1234567done\n"
         input = StringIO(server_bytes)
         output = StringIO()
         client_medium = medium.SmartSimplePipesClientMedium(input, output)
@@ -1741,7 +1770,7 @@
         smart_protocol.call('foo')
         smart_protocol.read_response_tuple(True)
         smart_protocol.cancel_read_body()
-        self.assertEqual(5, input.tell())
+        self.assertEqual(13, input.tell())
         self.assertRaises(
             errors.ReadingCompleted, smart_protocol.read_body_bytes)
 

=== modified file 'bzrlib/tests/test_wsgi.py'
--- bzrlib/tests/test_wsgi.py	2007-04-24 08:36:32 +0000
+++ bzrlib/tests/test_wsgi.py	2007-04-24 12:50:50 +0000
@@ -236,7 +236,7 @@
         response = self.read_response(iterable)
         self.assertEqual('200 OK', self.status)
         # Expect a version 2-encoded response.
-        self.assertEqual('2\nok\x012\n', response)
+        self.assertEqual('2\nsuccess\nok\x012\n', response)
 
 
 class FakeRequest(object):

